@@ -1,14 +1,20 @@
 //! The [`UtcOffset`] struct and its associated `impl`s.
 
+#[cfg(feature = "formatting")]
+use alloc::string::String;
+use core::cmp::Ordering;
 use core::fmt;
+use core::hash::{Hash, Hasher};
 use core::ops::Neg;
 #[cfg(feature = "formatting")]
 use std::io;
 
-use deranged::{RangedI32, RangedI8};
+use deranged::{RangedI8, RangedI32};
 use powerfmt::ext::FormatterExt;
 use powerfmt::smart_display::{self, FormatterOptions, Metadata, SmartDisplay};
 
+#[cfg(feature = "local-offset")]
+use crate::OffsetDateTime;
 use crate::convert::*;
 use crate::error;
 #[cfg(feature = "formatting")]
@@ -18,25 +24,24 @@ use crate::internal_macros::ensure_ranged;
 use crate::parsing::Parsable;
 #[cfg(feature = "local-offset")]
 use crate::sys::local_offset_at;
-#[cfg(feature = "local-offset")]
-use crate::OffsetDateTime;
 
 /// The type of the `hours` field of `UtcOffset`.
 type Hours = RangedI8<-25, 25>;
 /// The type of the `minutes` field of `UtcOffset`.
-type Minutes = RangedI8<{ -(Minute::per(Hour) as i8 - 1) }, { Minute::per(Hour) as i8 - 1 }>;
+type Minutes = RangedI8<{ -(Minute::per_t::<i8>(Hour) - 1) }, { Minute::per_t::<i8>(Hour) - 1 }>;
 /// The type of the `seconds` field of `UtcOffset`.
-type Seconds = RangedI8<{ -(Second::per(Minute) as i8 - 1) }, { Second::per(Minute) as i8 - 1 }>;
+type Seconds =
+    RangedI8<{ -(Second::per_t::<i8>(Minute) - 1) }, { Second::per_t::<i8>(Minute) - 1 }>;
 /// The type capable of storing the range of whole seconds that a `UtcOffset` can encompass.
 type WholeSeconds = RangedI32<
     {
-        Hours::MIN.get() as i32 * Second::per(Hour) as i32
-            + Minutes::MIN.get() as i32 * Second::per(Minute) as i32
+        Hours::MIN.get() as i32 * Second::per_t::<i32>(Hour)
+            + Minutes::MIN.get() as i32 * Second::per_t::<i32>(Minute)
             + Seconds::MIN.get() as i32
     },
     {
-        Hours::MAX.get() as i32 * Second::per(Hour) as i32
-            + Minutes::MAX.get() as i32 * Second::per(Minute) as i32
+        Hours::MAX.get() as i32 * Second::per_t::<i32>(Hour)
+            + Minutes::MAX.get() as i32 * Second::per_t::<i32>(Minute)
             + Seconds::MAX.get() as i32
     },
 >;
@@ -46,17 +51,96 @@ type WholeSeconds = RangedI32<
 /// This struct can store values up to ±25:59:59. If you need support outside this range, please
 /// file an issue with your use case.
 // All three components _must_ have the same sign.
-#[derive(Copy, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
+#[derive(Clone, Copy, Eq)]
+#[cfg_attr(not(docsrs), repr(C))]
 pub struct UtcOffset {
-    #[allow(clippy::missing_docs_in_private_items)]
-    hours: Hours,
-    #[allow(clippy::missing_docs_in_private_items)]
+    // The order of this struct's fields matter. Do not reorder them.
+
+    // Little endian version
+    #[cfg(target_endian = "little")]
+    seconds: Seconds,
+    #[cfg(target_endian = "little")]
     minutes: Minutes,
-    #[allow(clippy::missing_docs_in_private_items)]
+    #[cfg(target_endian = "little")]
+    hours: Hours,
+
+    // Big endian version
+    #[cfg(target_endian = "big")]
+    hours: Hours,
+    #[cfg(target_endian = "big")]
+    minutes: Minutes,
+    #[cfg(target_endian = "big")]
     seconds: Seconds,
 }
 
+impl Hash for UtcOffset {
+    #[inline]
+    fn hash<H>(&self, state: &mut H)
+    where
+        H: Hasher,
+    {
+        state.write_u32(self.as_u32_for_equality());
+    }
+}
+
+impl PartialEq for UtcOffset {
+    #[inline]
+    fn eq(&self, other: &Self) -> bool {
+        self.as_u32_for_equality().eq(&other.as_u32_for_equality())
+    }
+}
+
+impl PartialOrd for UtcOffset {
+    #[inline]
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for UtcOffset {
+    #[inline]
+    fn cmp(&self, other: &Self) -> Ordering {
+        self.as_i32_for_comparison()
+            .cmp(&other.as_i32_for_comparison())
+    }
+}
+
 impl UtcOffset {
+    /// Provide a representation of the `UtcOffset` as a `i32`. This value can be used for equality,
+    /// and hashing. This value is not suitable for ordering; use `as_i32_for_comparison` instead.
+    #[inline]
+    pub(crate) const fn as_u32_for_equality(self) -> u32 {
+        // Safety: Size and alignment are handled by the compiler. Both the source and destination
+        // types are plain old data (POD) types.
+        unsafe {
+            if const { cfg!(target_endian = "little") } {
+                core::mem::transmute::<[i8; 4], u32>([
+                    self.seconds.get(),
+                    self.minutes.get(),
+                    self.hours.get(),
+                    0,
+                ])
+            } else {
+                core::mem::transmute::<[i8; 4], u32>([
+                    self.hours.get(),
+                    self.minutes.get(),
+                    self.seconds.get(),
+                    0,
+                ])
+            }
+        }
+    }
+
+    /// Provide a representation of the `UtcOffset` as a `i32`. This value can be used for ordering.
+    /// While it is suitable for equality, `as_u32_for_equality` is preferred for performance
+    /// reasons.
+    #[inline]
+    const fn as_i32_for_comparison(self) -> i32 {
+        (self.hours.get() as i32) << 16
+            | (self.minutes.get() as i32) << 8
+            | (self.seconds.get() as i32)
+    }
+
     /// A `UtcOffset` that is UTC.
     ///
     /// ```rust
@@ -66,7 +150,6 @@ impl UtcOffset {
     /// ```
     pub const UTC: Self = Self::from_whole_seconds_ranged(WholeSeconds::new_static::<0>());
 
-    // region: constructors
     /// Create a `UtcOffset` representing an offset of the hours, minutes, and seconds provided, the
     /// validity of which must be guaranteed by the caller. All three parameters must have the same
     /// sign.
@@ -80,6 +163,8 @@ impl UtcOffset {
     /// While the signs of the parameters are required to match to avoid bugs, this is not a safety
     /// invariant.
     #[doc(hidden)]
+    #[inline]
+    #[track_caller]
     pub const unsafe fn __from_hms_unchecked(hours: i8, minutes: i8, seconds: i8) -> Self {
         // Safety: The caller must uphold the safety invariants.
         unsafe {
@@ -103,15 +188,16 @@ impl UtcOffset {
     /// assert_eq!(UtcOffset::from_hms(1, -2, -3)?.as_hms(), (1, 2, 3));
     /// # Ok::<_, time::Error>(())
     /// ```
+    #[inline]
     pub const fn from_hms(
         hours: i8,
         minutes: i8,
         seconds: i8,
     ) -> Result<Self, error::ComponentRange> {
         Ok(Self::from_hms_ranged(
-            ensure_ranged!(Hours: hours),
-            ensure_ranged!(Minutes: minutes),
-            ensure_ranged!(Seconds: seconds),
+            ensure_ranged!(Hours: hours("offset hour")),
+            ensure_ranged!(Minutes: minutes("offset minute")),
+            ensure_ranged!(Seconds: seconds("offset second")),
         ))
     }
 
@@ -119,6 +205,8 @@ impl UtcOffset {
     /// three parameters must have the same sign.
     ///
     /// While the signs of the parameters are required to match, this is not a safety invariant.
+    #[inline]
+    #[track_caller]
     pub(crate) const fn from_hms_ranged_unchecked(
         hours: Hours,
         minutes: Minutes,
@@ -149,6 +237,7 @@ impl UtcOffset {
     ///
     /// The sign of all three components should match. If they do not, all smaller components will
     /// have their signs flipped.
+    #[inline]
     pub(crate) const fn from_hms_ranged(
         hours: Hours,
         mut minutes: Minutes,
@@ -179,6 +268,7 @@ impl UtcOffset {
     /// assert_eq!(UtcOffset::from_whole_seconds(3_723)?.as_hms(), (1, 2, 3));
     /// # Ok::<_, time::Error>(())
     /// ```
+    #[inline]
     pub const fn from_whole_seconds(seconds: i32) -> Result<Self, error::ComponentRange> {
         Ok(Self::from_whole_seconds_ranged(
             ensure_ranged!(WholeSeconds: seconds),
@@ -196,19 +286,18 @@ impl UtcOffset {
     /// );
     /// # Ok::<_, time::Error>(())
     /// ```
+    #[inline]
     pub(crate) const fn from_whole_seconds_ranged(seconds: WholeSeconds) -> Self {
         // Safety: The type of `seconds` guarantees that all values are in range.
         unsafe {
             Self::__from_hms_unchecked(
-                (seconds.get() / Second::per(Hour) as i32) as _,
-                ((seconds.get() % Second::per(Hour) as i32) / Minute::per(Hour) as i32) as _,
-                (seconds.get() % Second::per(Minute) as i32) as _,
+                (seconds.get() / Second::per_t::<i32>(Hour)) as i8,
+                ((seconds.get() % Second::per_t::<i32>(Hour)) / Minute::per_t::<i32>(Hour)) as i8,
+                (seconds.get() % Second::per_t::<i32>(Minute)) as i8,
             )
         }
     }
-    // endregion constructors
 
-    // region: getters
     /// Obtain the UTC offset as its hours, minutes, and seconds. The sign of all three components
     /// will always match. A positive value indicates an offset to the east; a negative to the west.
     ///
@@ -217,6 +306,7 @@ impl UtcOffset {
     /// assert_eq!(offset!(+1:02:03).as_hms(), (1, 2, 3));
     /// assert_eq!(offset!(-1:02:03).as_hms(), (-1, -2, -3));
     /// ```
+    #[inline]
     pub const fn as_hms(self) -> (i8, i8, i8) {
         (self.hours.get(), self.minutes.get(), self.seconds.get())
     }
@@ -224,6 +314,7 @@ impl UtcOffset {
     /// Obtain the UTC offset as its hours, minutes, and seconds. The sign of all three components
     /// will always match. A positive value indicates an offset to the east; a negative to the west.
     #[cfg(feature = "quickcheck")]
+    #[inline]
     pub(crate) const fn as_hms_ranged(self) -> (Hours, Minutes, Seconds) {
         (self.hours, self.minutes, self.seconds)
     }
@@ -236,6 +327,7 @@ impl UtcOffset {
     /// assert_eq!(offset!(+1:02:03).whole_hours(), 1);
     /// assert_eq!(offset!(-1:02:03).whole_hours(), -1);
     /// ```
+    #[inline]
     pub const fn whole_hours(self) -> i8 {
         self.hours.get()
     }
@@ -248,8 +340,9 @@ impl UtcOffset {
     /// assert_eq!(offset!(+1:02:03).whole_minutes(), 62);
     /// assert_eq!(offset!(-1:02:03).whole_minutes(), -62);
     /// ```
+    #[inline]
     pub const fn whole_minutes(self) -> i16 {
-        self.hours.get() as i16 * Minute::per(Hour) as i16 + self.minutes.get() as i16
+        self.hours.get() as i16 * Minute::per_t::<i16>(Hour) + self.minutes.get() as i16
     }
 
     /// Obtain the number of minutes past the hour the offset is from UTC. A positive value
@@ -260,6 +353,7 @@ impl UtcOffset {
     /// assert_eq!(offset!(+1:02:03).minutes_past_hour(), 2);
     /// assert_eq!(offset!(-1:02:03).minutes_past_hour(), -2);
     /// ```
+    #[inline]
     pub const fn minutes_past_hour(self) -> i8 {
         self.minutes.get()
     }
@@ -274,9 +368,10 @@ impl UtcOffset {
     /// ```
     // This may be useful for anyone manually implementing arithmetic, as it
     // would let them construct a `Duration` directly.
+    #[inline]
     pub const fn whole_seconds(self) -> i32 {
-        self.hours.get() as i32 * Second::per(Hour) as i32
-            + self.minutes.get() as i32 * Second::per(Minute) as i32
+        self.hours.get() as i32 * Second::per_t::<i32>(Hour)
+            + self.minutes.get() as i32 * Second::per_t::<i32>(Minute)
             + self.seconds.get() as i32
     }
 
@@ -288,12 +383,11 @@ impl UtcOffset {
     /// assert_eq!(offset!(+1:02:03).seconds_past_minute(), 3);
     /// assert_eq!(offset!(-1:02:03).seconds_past_minute(), -3);
     /// ```
+    #[inline]
     pub const fn seconds_past_minute(self) -> i8 {
         self.seconds.get()
     }
-    // endregion getters
 
-    // region: is_{sign}
     /// Check if the offset is exactly UTC.
     ///
     ///
@@ -303,8 +397,9 @@ impl UtcOffset {
     /// assert!(!offset!(-1:02:03).is_utc());
     /// assert!(offset!(UTC).is_utc());
     /// ```
+    #[inline]
     pub const fn is_utc(self) -> bool {
-        self.hours.get() == 0 && self.minutes.get() == 0 && self.seconds.get() == 0
+        self.as_u32_for_equality() == Self::UTC.as_u32_for_equality()
     }
 
     /// Check if the offset is positive, or east of UTC.
@@ -315,8 +410,9 @@ impl UtcOffset {
     /// assert!(!offset!(-1:02:03).is_positive());
     /// assert!(!offset!(UTC).is_positive());
     /// ```
+    #[inline]
     pub const fn is_positive(self) -> bool {
-        self.hours.get() > 0 || self.minutes.get() > 0 || self.seconds.get() > 0
+        self.as_i32_for_comparison() > Self::UTC.as_i32_for_comparison()
     }
 
     /// Check if the offset is negative, or west of UTC.
@@ -327,12 +423,11 @@ impl UtcOffset {
     /// assert!(offset!(-1:02:03).is_negative());
     /// assert!(!offset!(UTC).is_negative());
     /// ```
+    #[inline]
     pub const fn is_negative(self) -> bool {
-        self.hours.get() < 0 || self.minutes.get() < 0 || self.seconds.get() < 0
+        self.as_i32_for_comparison() < Self::UTC.as_i32_for_comparison()
     }
-    // endregion is_{sign}
 
-    // region: local offset
     /// Attempt to obtain the system's UTC offset at a known moment in time. If the offset cannot be
     /// determined, an error is returned.
     ///
@@ -344,6 +439,7 @@ impl UtcOffset {
     /// # }
     /// ```
     #[cfg(feature = "local-offset")]
+    #[inline]
     pub fn local_offset_at(datetime: OffsetDateTime) -> Result<Self, error::IndeterminateOffset> {
         local_offset_at(datetime).ok_or(error::IndeterminateOffset)
     }
@@ -359,23 +455,23 @@ impl UtcOffset {
     /// # }
     /// ```
     #[cfg(feature = "local-offset")]
+    #[inline]
     pub fn current_local_offset() -> Result<Self, error::IndeterminateOffset> {
         let now = OffsetDateTime::now_utc();
         local_offset_at(now).ok_or(error::IndeterminateOffset)
     }
-    // endregion: local offset
 }
 
-// region: formatting & parsing
 #[cfg(feature = "formatting")]
 impl UtcOffset {
     /// Format the `UtcOffset` using the provided [format description](crate::format_description).
+    #[inline]
     pub fn format_into(
         self,
-        output: &mut impl io::Write,
+        output: &mut (impl io::Write + ?Sized),
         format: &(impl Formattable + ?Sized),
     ) -> Result<usize, error::Format> {
-        format.format_into(output, None, None, Some(self))
+        format.format_into(output, &self, &mut Default::default())
     }
 
     /// Format the `UtcOffset` using the provided [format description](crate::format_description).
@@ -387,8 +483,9 @@ impl UtcOffset {
     /// assert_eq!(offset!(+1).format(&format)?, "+01:00");
     /// # Ok::<_, time::Error>(())
     /// ```
+    #[inline]
     pub fn format(self, format: &(impl Formattable + ?Sized)) -> Result<String, error::Format> {
-        format.format(None, None, Some(self))
+        format.format(&self, &mut Default::default())
     }
 }
 
@@ -404,6 +501,7 @@ impl UtcOffset {
     /// assert_eq!(UtcOffset::parse("-03:42", &format)?, offset!(-3:42));
     /// # Ok::<_, time::Error>(())
     /// ```
+    #[inline]
     pub fn parse(
         input: &str,
         description: &(impl Parsable + ?Sized),
@@ -413,6 +511,7 @@ impl UtcOffset {
 }
 
 mod private {
+    /// Metadata for `UtcOffset`.
     #[non_exhaustive]
     #[derive(Debug, Clone, Copy)]
     pub struct UtcOffsetMetadata;
@@ -422,7 +521,8 @@ use private::UtcOffsetMetadata;
 impl SmartDisplay for UtcOffset {
     type Metadata = UtcOffsetMetadata;
 
-    fn metadata(&self, _: FormatterOptions) -> Metadata<Self> {
+    #[inline]
+    fn metadata(&self, _: FormatterOptions) -> Metadata<'_, Self> {
         let sign = if self.is_negative() { '-' } else { '+' };
         let width = smart_display::padded_width_of!(
             sign,
@@ -435,6 +535,7 @@ impl SmartDisplay for UtcOffset {
         Metadata::new(width, self, UtcOffsetMetadata)
     }
 
+    #[inline]
     fn fmt_with_metadata(
         &self,
         f: &mut fmt::Formatter<'_>,
@@ -454,21 +555,23 @@ impl SmartDisplay for UtcOffset {
 }
 
 impl fmt::Display for UtcOffset {
+    #[inline]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         SmartDisplay::fmt(self, f)
     }
 }
 
 impl fmt::Debug for UtcOffset {
+    #[inline]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt::Display::fmt(self, f)
     }
 }
-// endregion formatting & parsing
 
 impl Neg for UtcOffset {
     type Output = Self;
 
+    #[inline]
     fn neg(self) -> Self::Output {
         Self::from_hms_ranged(self.hours.neg(), self.minutes.neg(), self.seconds.neg())
     }
